#include <UnitTest++/UnitTest++.h>

#include "sample.h"
#include "sampleset.h"

#include <algorithm>
#include <iostream>
#include <sstream>
#include <vector>

// A utility to get the sorted sizes of the clusters
// for easy comparison in a test
void storeSortedSizes( const SampleSet &samples, std::vector<int> &sizes )
{
  for (int i = 0; i < samples.clusterCount(); ++i) {
    sizes.push_back( samples.clusterSize( i ) );
  }
  std::sort( sizes.begin(), sizes.end() );
}

TEST(readSample)
{
  Sample sample1( 5 ), sample2( 5 );

  CHECK_EQUAL( 5, sample1.size() );
  CHECK_EQUAL( -1, sample1.id );

  std::istringstream in("7 4.2 0.2 -5 16 8.1\n9 3.1 2.6 0.0 18 9.2");
  in >> sample1 >> sample2;
  CHECK_EQUAL( 7, sample1.id );
  CHECK_EQUAL( 4.2, sample1.data[0] );
  CHECK_EQUAL( 8.1, sample1.data[4] );
  CHECK_EQUAL( 9, sample2.id );
  CHECK_EQUAL( 3.1, sample2.data[0] );
  CHECK_EQUAL( 9.2, sample2.data[4] );
  CHECK_CLOSE( 6.0975, sample1.distance( sample2 ), 0.0001 );
  CHECK_CLOSE( 6.0975, sample2.distance( sample1 ), 0.0001 );
}

TEST(readXuToyData)
{
  SampleSet samples( 2 );
  CHECK_EQUAL( 17, samples.readFile( "toy-data.txt" ) );
  CHECK_EQUAL( 17, samples.size() );
  Sample first = samples[ 0 ];
  CHECK_EQUAL( 1, first.id );
  CHECK_EQUAL( 2, first.size() );
  CHECK_EQUAL( 8.8, first.data[0] );
  CHECK_EQUAL( 24.5, first.data[1] );

  Sample last = samples[ 16 ];
  CHECK_EQUAL( 17, last.id );
  CHECK_EQUAL( 2, last.size() );
  CHECK_EQUAL( 24.9, last.data[0] );
  CHECK_EQUAL( 4.5, last.data[1] );
}

TEST(clusterXuToyData)
{
  SampleSet samples( 2 );
  CHECK_EQUAL( 0, samples.size() );
  CHECK_EQUAL( 0, samples.clusterCount() );
  samples.readFile( "toy-data.txt" );
  CHECK_EQUAL( 17, samples.size() );
  CHECK_EQUAL( 17, samples.clusterCount() );
  CHECK_EQUAL( 4, samples.cluster() );
  CHECK_EQUAL( 4, samples.clusterCount() );
  std::vector<int> sizes;
  storeSortedSizes( samples, sizes );
  CHECK_EQUAL( 4, sizes[ 0 ] );
  CHECK_EQUAL( 4, sizes[ 1 ] );
  CHECK_EQUAL( 4, sizes[ 2 ] );
  CHECK_EQUAL( 5, sizes[ 3 ] );
}

TEST(clusterRandomData)
{
  SampleSet samples( 5 );
  CHECK_EQUAL( 100, samples.readFile( "random-data.txt" ) );
  CHECK_EQUAL( samples.size(), samples.clusterCount() );
  CHECK_EQUAL( 3, samples.cluster() );
  std::vector<int> sizes;
  storeSortedSizes( samples, sizes );
  CHECK_EQUAL( 31, sizes[ 0 ] );
  CHECK_EQUAL( 34, sizes[ 1 ] );
  CHECK_EQUAL( 35, sizes[ 2 ] );
}

int main()
{
    return UnitTest::RunAllTests();
}
